/*
 * $Id: AutoTransformer.java 11604 2008-04-18 22:39:51Z rossmason $
 * --------------------------------------------------------------------------------------
 * Copyright (c) MuleSource, Inc.  All rights reserved.  http://www.mulesource.com
 *
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.transformer.simple;

import org.mule.transformer.AbstractMessageAwareTransformer;
import org.mule.api.MuleMessage;
import org.mule.api.lifecycle.InitialisationException;
import org.mule.api.transformer.TransformerException;
import org.mule.config.i18n.CoreMessages;

/**
 * A transformer that uses the transform discovery mechanism to convert the message payload. This transformer
 * works much better when transforming custom object types rather that java types since there is less chance for
 * ambiguity.
 * If an exact match cannot be made an execption will be thrown.
 */
public class AutoTransformer extends AbstractMessageAwareTransformer
{
    /**
     * Template method where deriving classes can do any initialisation after the
     * properties have been set on this transformer
     *
     * @throws org.mule.api.lifecycle.InitialisationException
     *
     */
    //@Override
    public void initialise() throws InitialisationException
    {
        super.initialise();
        if(getReturnClass().equals(Object.class))
        {
            throw new InitialisationException(CoreMessages.transformerInvalidReturnType(Object.class, getName()), this);
        }
    }

    public Object transform(MuleMessage message, String outputEncoding) throws TransformerException
    {
        return message.getPayload(getReturnClass());
    }
}
